//    .----------------------------------------------------------------------------------------------------------------------------------------------------------------.
//    |                                                                    MagLoop-Stepper withDC-feeder                                                               |
//    '----------------------------------------------------------------------------------------------------------------------------------------------------------------'
//     by Daniel Wendt-Fröhlich, DL2AB (BugReports/Changes to danielwf@hackerspace-bremen.de, dl2ab@darc.de) for https://funkfreun.de
//     License CC-by-SA 3.0 - 02/2023-06/2023 - Bremen(GER) -- http://creativecommons.org/licenses/by-sa/3.0/de/ 
//     Circuit and further informations: https://wiki.funkfreun.de/projekte/magloop-dl2ab

// Source Motorcontrol: https://github.com/NikodemBartnik/ArduinoTutorials/tree/master/28BYJ-48 and https://www.roboter-bausatz.de/projekte/schrittmotor-mit-arduino-ansteuern

// Configuration:
#define WAIT_TIME 500     // time between fine step and continous drive
#define FINESTEPS_LEFT 10  // motor steps for left fine step
#define FINESTEPS_RIGHT 20 // motor steps for right fine step

// Connections: 
#define STEPPER_PIN_1 8      // Connections for stepper motor driver ULN2003
#define STEPPER_PIN_2 9
#define STEPPER_PIN_3 10
#define STEPPER_PIN_4 11
#define VOLT_PIN A2          // Connection for voltage sensing, for up to 15V:  VCC--[Resistor 2.2k]--AnalogIn--[Resistor 1.0K]--GND

//Voltage Levels (max Voltage per Button x 100)
//Example: 12V(VCC) -- 10V(Level1) -- Button1 -- 9V(Level2) -- Button 2 -- 8V(Level3) -- Button 1+2 -- 7V(Level4)
// Use Serial Monitor at 9600 to read the ADC-Levels for different buttons
#define VOLTAGE_LEVEL_1 1000
#define VOLTAGE_LEVEL_2 900
#define VOLTAGE_LEVEL_3 800
#define VOLTAGE_LEVEL_4 700


byte step_number = 0;
byte button = 0;
byte old_button = 0;
unsigned long button_time = 0;
unsigned long debounce_time = 0;
unsigned long serial_time = 0;

void setup() {
  pinMode(STEPPER_PIN_1, OUTPUT);
  pinMode(STEPPER_PIN_2, OUTPUT);
  pinMode(STEPPER_PIN_3, OUTPUT);
  pinMode(STEPPER_PIN_4, OUTPUT);
  analogReadResolution(10);
  Serial.begin(9600);
}

void loop() {

  buttonRead();

  if (old_button != button) {
    button_time = millis() + WAIT_TIME;
    if (button == 0) {
      digitalWrite(STEPPER_PIN_1, LOW);
      digitalWrite(STEPPER_PIN_2, LOW);
      digitalWrite(STEPPER_PIN_3, LOW);
      digitalWrite(STEPPER_PIN_4, LOW);
      if (old_button != 0) {
        digitalWrite(VOLT_PIN, HIGH); 
        delay(20);
        digitalWrite(VOLT_PIN, LOW); 
      }
    }  
    if (button == 1) {
      for (int i=0; i < FINESTEPS_LEFT; i++){
        OneStep(false);
        delay(30);
      }
    }
    if (button == 2) {
      for (int i=0; i < FINESTEPS_RIGHT; i++){
        OneStep(true);
        delay(30);
      }
    }
  }
  
  if ( (millis() > button_time) && (old_button == button) ) {
    if (button == 0) {
      digitalWrite(STEPPER_PIN_1, LOW);
      digitalWrite(STEPPER_PIN_2, LOW);
      digitalWrite(STEPPER_PIN_3, LOW);
      digitalWrite(STEPPER_PIN_4, LOW);
      if (old_button != 0) {
        digitalWrite(VOLT_PIN, HIGH); 
        delay(20);
        digitalWrite(VOLT_PIN, LOW); 
      } 
    }  
    if (button == 1) {
      OneStep(false);
      delay(2);
    }
    if (button == 2) {
      OneStep(true);
      delay(2);
    }
        
  }


  
}

void buttonRead() {
  old_button = button;
  int button_voltage = analogRead(VOLT_PIN) * 1.562;  //short for: 5V / 1024steps * 3,2KOhm(R1+R2) / 1KOhm(R2) * 100
  byte button_temp = 0;
  if ( ( button_voltage > VOLTAGE_LEVEL_1 ) || ( button_voltage < VOLTAGE_LEVEL_4 ) ) button_temp = 0;
  if ( ( button_voltage <= VOLTAGE_LEVEL_1 ) && ( button_voltage > VOLTAGE_LEVEL_2 ) ) button_temp = 1;
  if ( ( button_voltage <= VOLTAGE_LEVEL_2 ) && ( button_voltage > VOLTAGE_LEVEL_3 ) ) button_temp = 2;
  if ( ( button_voltage <= VOLTAGE_LEVEL_3 ) && ( button_voltage > VOLTAGE_LEVEL_4 ) ) button_temp = 3;
  if ( (button_temp != button) && (millis() < debounce_time) ) debounce_time = millis() + 2000;
  if ( (millis() >= debounce_time) && (button_temp != button) ) button = button_temp;
    
  if ( millis() >= serial_time ) {  
    Serial.print(button_voltage); 
    Serial.print(" /100 V - Button pressed: ");
    Serial.println(button);
    serial_time = millis() + 500;
  }
  
}


void OneStep(bool dir){
  if(dir){
    switch(step_number){
      case 0:
        digitalWrite(STEPPER_PIN_1, HIGH);
        digitalWrite(STEPPER_PIN_2, LOW);
        digitalWrite(STEPPER_PIN_3, LOW);
        digitalWrite(STEPPER_PIN_4, LOW);
      break;
      case 1:
        digitalWrite(STEPPER_PIN_1, LOW);
        digitalWrite(STEPPER_PIN_2, HIGH);
        digitalWrite(STEPPER_PIN_3, LOW);
        digitalWrite(STEPPER_PIN_4, LOW);
      break;
      case 2:
        digitalWrite(STEPPER_PIN_1, LOW);
        digitalWrite(STEPPER_PIN_2, LOW);
        digitalWrite(STEPPER_PIN_3, HIGH);
        digitalWrite(STEPPER_PIN_4, LOW);
      break;
      case 3:
        digitalWrite(STEPPER_PIN_1, LOW);
        digitalWrite(STEPPER_PIN_2, LOW);
        digitalWrite(STEPPER_PIN_3, LOW);
        digitalWrite(STEPPER_PIN_4, HIGH);
      break;
    } 
  }else{
    switch(step_number){
      case 0:
        digitalWrite(STEPPER_PIN_1, LOW);
        digitalWrite(STEPPER_PIN_2, LOW);
        digitalWrite(STEPPER_PIN_3, LOW);
        digitalWrite(STEPPER_PIN_4, HIGH);
      break;
      case 1:
        digitalWrite(STEPPER_PIN_1, LOW);
        digitalWrite(STEPPER_PIN_2, LOW);
        digitalWrite(STEPPER_PIN_3, HIGH);
        digitalWrite(STEPPER_PIN_4, LOW);
      break;
      case 2:
        digitalWrite(STEPPER_PIN_1, LOW);
        digitalWrite(STEPPER_PIN_2, HIGH);
        digitalWrite(STEPPER_PIN_3, LOW);
        digitalWrite(STEPPER_PIN_4, LOW);
      break;
      case 3:
        digitalWrite(STEPPER_PIN_1, HIGH);
        digitalWrite(STEPPER_PIN_2, LOW);
        digitalWrite(STEPPER_PIN_3, LOW);
        digitalWrite(STEPPER_PIN_4, LOW);
    } 
  }
  
  step_number++;
  if(step_number > 3){
    step_number = 0;
  }
}
